/***************************************************************************
                          common.h  -  description
                             -------------------
    begin                : 02.04.04
    copyright            : (C) 2004 by Pablo Yanez Trujillo
    email                : rex@supertux.homelinux.org
    Version			: 1.0.2
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COMMON_H
#define COMMON_H

#include <sys/types.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>

/* variables that shouldn't be global :) */
extern FILE* output;

/* Some useful constants */

#define MAX_LENGTH 4096
#define MAX_PATH 500
#define PACKAGE "Generate Background genbg"
#define VERSION "1.0.2"


/* typedef with information about the command line and configuration file*/

typedef enum {
	ARG_HELP=0,
	ARG_VERSION=1,
	ARG_CONFIG=2,
	ARG_PATH=3,
	ARG_INCLUDE=4,
	ARG_ERROR=-1
} ARG_TYPE;

typedef enum {
	CONF_DEFAULT=0,
	CONF_DEBUG=1,
	CONF_BUFFER=2,
	CONF_FILETYPE=3,
	CONF_TOOL=4,
	CONF_OPTIONS=5,
	CONF_BLANK=6,
	CONF_COMMENT=7,
	CONF_ERROR=-1
} CONF_TYPE;

typedef struct {
	int i[2]; /* the first flag indicates that i is set. The second flag tells where. */
	char default_dir[10][MAX_LENGTH]; /* an char* array with the default directories */
	size_t def_count;
	char (*type)[10]; /* type to search */
	size_t max_type_length;
	size_t type_count;
	int debug; /* for debug modus -1 is not set, 0 is no debug, 1 is debug */ 
	/* I don't add a FILE* outputbuffer because I made a global variable */
	char tool[MAX_PATH]; /* path to the tool. as default set to wmsetbg */
	char options[MAX_LENGTH]; /* options for tool*/
} INFO;


/* tools functions */

void help(const char* progname);
void version();
void configure(int flag); /*if flag is 1 then configure will ask */
char* cutNL(char* buffer); /* cuts the \n from the line */
char* ltrim(char* dest, const char* src);
char* rtrim(char* dest, const char* src);
char* trim(char* dest, const char* src);

/*
	the functions ltrim, rtrim, trim cut the blank spaces (" \t\r") from the line
	and return a pointer to a char. This functions don't use malloc. They copy the
	the new line in dest and return it.
	
	ltrim cuts all blank characters in the left side.
	rtrim cuts all blank characters in the right side.
	trim cuts all blank character in the left and right side.

	if src isn't a 0-terminated string it is not determinated what the function will do.

*/

ARG_TYPE get_ArgType(const char* line);
int parse_args(int argc, char** argv, INFO* info);
CONF_TYPE get_ConfType(char** line);

/*
	This function cuts the first part of the option unless it is a comment or a blank line.
	the string will be trimed and "line" will be set to the first character of the option
*/

int parse_conf_file(INFO* info);
int getNumTypes(const char* line);
int isPicture(const char* path, INFO* info);
int generateRandom(int left, int right);


#endif
