/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : 28.03.04
    copyright            : (C) 2004 by Pablo Yanez Trujillo
    email                : rex@supertux.homelinux.org
    Version			: 1.0.2
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Xresources.h"
#include "common.h"
#include "sll.h"
#include "db.h"
#include "debugoutput.h"

/* check code for debug==0 no printout  */
int main(int argc, char** argv)
{
	Display* display;
	int i,id, height, width;
	TREE* tree;
	char command [MAX_LENGTH], *tmp;

	INFO info;
	/* setting default values */
	info.def_count=0;
	info.type_count=0;
	info.max_type_length=0;
	info.debug=-1;
	info.tool[0]=0;
	info.options[0]=0;
	output=NULL;
	/* parsing arguemnt */
	if (!parse_args(argc, argv, &info)) {
		fprintf(stderr, "Unexpected error while parsing the command line arguments.\n");
		return EXIT_FAILURE;
	}

	if (!parse_conf_file(&info)) {
		fprintf(stderr, "Error while parsing configure file.\n");
		return EXIT_FAILURE;
	}

	display = NULL;
	display = connectX(display);
	get_size(display, &width, &height);

	tree = createTREE(&tree);
	if (!tree) return EXIT_SUCCESS;

	id=1; /* this is important*/
  
	if (info.i[0]) {
		if (argc==2 && !strcmp(argv[1], "-i")) 
			add_files(tree, ".", &id, 0, &info);
		if (argc==1 && info.def_count==0)  /* no arguments, -i set as default */
			add_files(tree, ".", &id, 0, &info);
		else {	
			for (i=0; (unsigned) i<info.def_count; ++i)
				add_files(tree, info.default_dir[i], &id, 0, &info);
		}
	}
    	
	for(i=1; i<argc; ++i) {
		if ((info.i[0] && info.i[1]!=i) || info.i[1]==-1)
		add_files(tree, argv[i], &id,0, &info);
	}

	--id;

	if (tree->empty) {
		dfprintf(output, "There are no files that has been added\n");
		killTREE(tree);
		closeX(display);
		return EXIT_FAILURE;
	}

	dfprintf(output, "%d files added into the database.\n", id);
    
	do {
		i=generateRandom(1, id);
		tmp = getname(tree, i);
	} while (tmp==NULL);

	dfprintf(output, "Selected picture: %s (id=%d)\n", tmp, i);

	sprintf(command, "%s %s \"%s\"",info.tool, info.options, tmp);
	dfprintf(output, "Executing... %s\n", command);
	system(command);
    
	free(info.type);
    
	if (output!=stdout && output!=stderr) {
		dfprintf(output, "# -------------------------- end of debug informtion --------------------------------\n");
		if (output) 
			fclose(output);
	}

	killTREE(tree);
	closeX(display);
	return EXIT_SUCCESS;
}
