/***************************************************************************
                          tools.c  -  description
                             -------------------
    begin                : 06.04.2004
    copyright            : (C) 2004 by Pablo Yanez Trujillo
    email                : rex@supertux.homelinux.org
    Version			: 1.0.2
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <time.h>
#include "common.h"

FILE* output;

void help(const char* progname)
{
	printf("usage: %s [--config] [--help] [--version] [-i] [directory 1] [directory 2] ...\n\n\
--config\tCreates configure file ~/.genbgrc\n\
--help\t\tShows this help\n\
--version\tShows the version\n\
-i\t\t\tincludes default directories from the configure file. For more\n\
\t\t\tinformation type man genbg or man genbgrc.\n\
directory x\tList of directories in which genbg will search for graphic files\n\n\
Bugs reports: rex@supertux.homelinux.org or pabloy@pcpool02.mathematik.uni-freiburg.de\n", progname);

}

void version()
{
	printf("GNU Generate Background Version %s\n", VERSION);
}

void configure(int flags)
{
	char def_dir[MAX_LENGTH];
	char type[MAX_LENGTH], type_input[MAX_LENGTH];
	char debug[MAX_LENGTH];
	char deb_buff[MAX_LENGTH];
	char tools[MAX_LENGTH];
	char options[MAX_LENGTH];
	char* path;
	char filename[MAX_LENGTH];
	FILE* fbuffer;	
	
	/* default values */
	strcpy(type, ".jpg .gif .bmp .png .xpm");
	strcpy(debug, "on");
	strcpy(tools, "wmsetbg");
	*options=0;
	
	if (NULL==(path=getenv("HOME"))) {
		fprintf(stderr, "Your $HOME Variable is not set!\n");
		exit(EXIT_FAILURE);
	}
	
	sprintf(filename, "%s%s.genbgrc", path, path[strlen(path)-1]=='/' ? "" : "/");
	
	if (NULL==(fbuffer = fopen(filename, "w"))) {
		fprintf(stderr, "%s cannot be opened\n", filename);
		exit(EXIT_FAILURE);
	}
	
	fprintf(fbuffer, "# %s created by genbg version %s\n\n", filename, VERSION);

	if (flags) {
		fprintf(fbuffer, "# default directories\n");
		do {
			printf("Insert the path to the default directories (Press only ENTER to finish)\n");
			fgets(def_dir, MAX_LENGTH, stdin);
			if (!strcmp(def_dir, "\n")) *def_dir=0;
			cutNL(def_dir);
			if (*def_dir) 
				fprintf(fbuffer, "default=%s\n", def_dir);
		} while(strcmp(def_dir, ""));
	}
	
	fprintf(fbuffer, "\n");
	
	if (flags) {
		do {
			printf("Do you want genbg to run in debug modues? ([on], off): ");
			fgets(debug, MAX_LENGTH, stdin);
			trim(debug, debug);
			if (!*debug) strcpy(debug, "on");
		} while (strcmp(debug, "on") && strcmp(debug, "off"));
	}
	
	fprintf(fbuffer, "#debug modues\n");
	fprintf(fbuffer, "DEBUG=%s\n", debug);
	
	// choose buffer!
	if (!strcmp(debug, "on")) {
		if (flags) {
			printf("Insert the path to the debug logfile or the output buffer: ([STDOUT], STDERR, path to file)\n");
			fgets(deb_buff, MAX_LENGTH, stdin);
			trim(deb_buff, deb_buff);
			if (!*deb_buff) strcpy(deb_buff, "STDOUT");
		} else strcpy(deb_buff, "STDOUT");
		
		fprintf(fbuffer, "ERR_BUFFER=%s\n", deb_buff);
	
	}
	
	fprintf(fbuffer, "\n");
	
	if (flags) {
		*type=0;
		do {
			printf("Insert the file type to search for (example: .jpg) Note, this is case sensitive\n");
			printf("Press only ENTER to finish: ");
			fgets(type_input, MAX_LENGTH, stdin);
			trim(type_input, type_input);
			strcat(type, " ");
			strcat(type, type_input);
		} while (strcmp(type_input, ""));
		ltrim(type, type);
	}
	
	fprintf(fbuffer, "# type file to search for\n");
	if (!*type) strcpy(type, ".jpg .gif .bmp .png .xpm");
	fprintf(fbuffer, "TYPE=%s\n", type);
	
	fprintf(fbuffer, "\n");
	
	if (flags) {
		printf("Insert the tool you want to use to set the background. ([wmsetbg], your tool): ");
		fgets(tools, MAX_LENGTH, stdin);
		trim(tools, tools);
		if (!*tools) strcpy(tools, "wmsetbg");
	}
	
	fprintf(fbuffer, "# setting tool\n");
	fprintf(fbuffer, "TOOL=%s\n", tools);
	
	fprintf(fbuffer, "\n");
	
	if (flags) {
		printf("You may have some options for the execution of %s. Please, insert them:", tools);
		fgets(options, MAX_LENGTH, stdin);
		trim(options, options);
		if (*options) fprintf(fbuffer, "OPTIONS=%s\n", options);
	}
	
	fprintf(fbuffer, "\n# ------------ eof ------------------");
	
	fclose(fbuffer);
	
	if (flags) {
		printf("\ngenbg has been configured successfully\n");
	}
}

ARG_TYPE get_ArgType(const char* line)
{
	if (!line)
		return ARG_ERROR;
		
	if (!strcmp(line, "--help"))
		return ARG_HELP;
	
	if (!strcmp(line, "--config"))
		return ARG_CONFIG;
		
	if (!strcmp(line, "--version"))
		return ARG_VERSION;
		
	if (line[0]=='-') {
		if (line[1]=='i') {
			return ARG_INCLUDE;
		} else return ARG_ERROR;
	}
		
	/* ok, no other arguments that I can parse
	   the rest of the arguments may be file name */
	   
	return ARG_PATH;
	
}

int parse_args(int argc, char** argv, INFO* info)
{
	int i;
	if (argc<0 || !argv || !info) return 0;
	info->i[0]=0;
	info->i[1]=-1;
	if (argc==1) {
		info->i[0]=1;
		info->i[1]=0;
		return 1;
	}
	for(i=1; i<argc; ++i) {
		switch(get_ArgType(argv[i])) {
			case ARG_HELP:
				help(argv[0]);
				exit(EXIT_SUCCESS);
			case ARG_VERSION:
				version();
				exit(EXIT_SUCCESS);
			case ARG_CONFIG:
				configure(1);
				exit(EXIT_SUCCESS);
			case ARG_INCLUDE:
				info->i[0]=1;
				info->i[1]=i;
				break;
			case ARG_ERROR:
				fprintf(stderr, "%s argument is invalid\n", argv[i]);
				exit(EXIT_FAILURE);
			case ARG_PATH:
			default: /* do nothing */
				break;
		}
	}
	
	return 1;
}


CONF_TYPE get_ConfType(char** line)
{
	char command[MAX_LENGTH], *tmp, *lineptr=*line;
	int len;
	if (!line || !*line) return CONF_ERROR;

	trim(lineptr, lineptr);
	
	if (*lineptr=='#')
		return CONF_COMMENT;
	if (!*lineptr)
		return CONF_BLANK;
		
	if (NULL==(tmp=strstr(lineptr, "="))) {
		fprintf(stderr, "%s command is not set. Missing '=': Bad rvalue\n", lineptr);
		return CONF_ERROR;
	}
	
	if (tmp==lineptr) {
		fprintf(stderr, "Bad lvalue at '%s'\n", tmp);
		return CONF_ERROR;
	}
	
	len = tmp-lineptr;
	if (len>MAX_LENGTH-1) {
		fprintf(stderr, "The line is to large. The line may have max. %d characters\n", MAX_LENGTH-1);
		return CONF_ERROR;		
	}
	strncpy(command, lineptr, len);
	command[len]=0;
	
	tmp++;
	*line=tmp;
	rtrim(command, command);

	if(!*tmp) {
		fprintf(stderr, "%s command is not set. Missing rvalue after '='\n", lineptr);
		return CONF_ERROR;
	}
		
	if (!strcmp(command, "default"))
		return CONF_DEFAULT;
	if (!strcmp(command, "DEBUG"))
		return CONF_DEBUG;
	if (!strcmp(command, "ERR_BUFFER"))
		return CONF_BUFFER;
	if (!strcmp(command, "TYPE"))
		return CONF_FILETYPE;
	if (!strcmp(command, "TOOL"))
		return CONF_TOOL;
	if (!strcmp(command, "OPTIONS"))
		return CONF_OPTIONS;
	
	fprintf(stderr, "'%s': unknown command\n", command);
	return CONF_ERROR;
}

int parse_conf_file(INFO* info)
{
	FILE* fbuffer;
	char filename[MAX_LENGTH], *path, line[MAX_LENGTH], *tmp, *tokens, (*type_tmp)[10];
	CONF_TYPE conf_type;
	int counter, type_count;
	int i;

	if (!info) return 0;
	
	if (NULL==(path=getenv("HOME"))) {
		fprintf(stderr, "Your $HOME Variable is not set!\n");
		return 0;
	}
	
	sprintf(filename, "%s%s.genbgrc", path, path[strlen(path)-1]=='/' ? "" : "/");
	
	if (NULL==(fbuffer = fopen(filename, "r"))) {
		/* try to write default configuration */
		configure(0);
	}
	
	if (NULL==(fbuffer = fopen(filename, "r"))) {
		fprintf(stderr, "%s cannot be opened\n", filename);
		return 0;
	}
	counter=0;
	while ((NULL!=(fgets(line, MAX_LENGTH, fbuffer))) && !feof(fbuffer) && !ferror(fbuffer)) {
		++counter;
		cutNL(line);
		tmp=line;
		conf_type=get_ConfType(&tmp);
		ltrim(tmp, tmp);
		switch(conf_type) {
			case CONF_DEFAULT:
				if (info->def_count < 10) {
					strcpy(info->default_dir[info->def_count], tmp);
					info->def_count++;
				}				
				break;
			case CONF_DEBUG:
				if (output!=NULL) {
					/* ERR_BUFFER is done before debug. Setting debug to on */
					info->debug=1;
					break;
				}
				if (strcmp(tmp, "on") && strcmp(tmp, "off")) {
					fprintf(stderr, "On line %d is lvalue '%s' for DEBUG invalid.\n", counter, tmp);
					return 0;
				}
				switch(tmp[1]) {
					case 'n':
						info->debug=1;
						break;
					case 'f':
						info->debug=0;
				}
				break;
			case CONF_BUFFER:
				if (info->debug==0) /* skip this */
					break;
				
				if (!strcmp(tmp, "STDOUT")) {
					output=stdout;
					break;
				}
				
				if (!strcmp(tmp, "STDERR")) {
					output=stderr;
				}
				
				if (NULL==(output=fopen(tmp, "a"))) {
					fprintf(stderr, "%s cannot be opened. Cannot write debug messages.\n", tmp);
					fprintf(stderr, "Setting stdout as default output stream.\n");
					output=stdout;
				}				
				break;
			case CONF_FILETYPE:
				type_count = getNumTypes(tmp);
				if (NULL==(type_tmp=malloc(sizeof(char[10])*type_count))) {
					fprintf(stderr, "Error allocating the memory\n");
					return 0;
				}
				i=0;
				tokens = strtok(tmp, " \f\n\r\t\v"); /* token cannot be NULL */

				if (strlen(tokens)<9) {
					if (*tokens=='.') 
						strcpy(type_tmp[i++], tokens);
					else {
						strcpy(type_tmp[i++], ".");
						strcat(type_tmp[i-1], tokens);
					}
				}
				info->max_type_length=strlen(type_tmp[0]);
				while (NULL!=(tokens=strtok(NULL, " \f\n\r\t\v"))) {
					if (strlen(tokens)<9) {
						if (*tokens=='.') 
							strcpy(type_tmp[i++], tokens);
						else {
							strcpy(type_tmp[i++], ".");
							strcat(type_tmp[i-1], tokens);
						}
					}
					info->max_type_length=info->max_type_length > strlen(type_tmp[i-1]) ? info->max_type_length : strlen(type_tmp[i-1]);
				}
				info->type=type_tmp;
				info->type_count=i;
				break;
			case CONF_TOOL:
				strcpy(info->tool, tmp);
				break;
			case CONF_OPTIONS:
				strcpy(info->options, tmp);
				break;
			case CONF_BLANK:
			case CONF_COMMENT:
				break;
			case CONF_ERROR:
				fprintf(stderr, "On line %d '%s'\n", counter, line);
				return 0;
		}
	}
	
	if (ferror(fbuffer)) {
		fprintf(stderr, "Unexpected error reading ~/.genbgrc\n");
		return 0;
	}
	
	/* verify which variables are set */
	if (info->type_count==0) {
		if (NULL==(type_tmp = malloc(sizeof(char[10])*5))) {
			fprintf(stderr, "Error allocating memory\n");
			return 0;
		}
		strcpy(type_tmp[0], ".jpg");
		strcpy(type_tmp[1], ".gif");
		strcpy(type_tmp[2], ".bmp");
		strcpy(type_tmp[3], ".png");
		strcpy(type_tmp[4], ".xpm");
		info->type=type_tmp;
		info->type_count=5;
		info->max_type_length=4;
	}
	
	if (info->debug==-1) {
		info->debug=1;
		output=stdout;
	}
	
	if (info->tool[0]==0) {
		strcpy(info->tool, "wmsetbg");
		info->options[0]=0;
	}
	
	if (output==NULL && info->debug==1) 
		output=stdout;
	
	fclose(fbuffer);
	return 1;
}

char* cutNL(char* buffer)
{
	int s = strlen(buffer);
	if (!buffer) return NULL;
	if (buffer[s-1]=='\n')
		buffer[s-1]=0;
	return buffer;
}

char* rtrim(char* dest, const char* src)
{
	int s=strlen(src);
	char src_cpy[MAX_LENGTH], *tmp;
	if (!dest || !src) return dest=NULL;
	strcpy(src_cpy,src);
	tmp=src_cpy;
	tmp+=s;
	
	if (tmp==src_cpy) { *dest=0; return dest;  }
	tmp--;s--;

	while(s-- && (isspace(*tmp)&&isspace(*(tmp-1))) ) {
		*tmp=0;
		tmp--;
	}
	if (isspace(*tmp)) *tmp=0;

	strcpy(dest, src_cpy);

	return dest;
}

char* ltrim(char* dest, const char* src)
{
	int s=strlen(src);
	char src_cpy[MAX_LENGTH], *tmp;
	if (!dest || !src) return dest=NULL;
	strcpy(src_cpy,src);
	tmp=src_cpy;
	if (!*src) { *dest=0; return dest;  }
	s--;

	while(s-- && (isspace(*tmp)&&isspace(*(tmp+1))) )
		tmp++;
	if (isspace(*tmp)) tmp++;

	strcpy(dest, tmp);

	return dest;
}

char* trim(char* dest, const char* src)
{
	if (!dest || !src) return dest=NULL;
	ltrim(dest, src);
	rtrim(dest, dest);
	return dest;
}

int getNumTypes(const char* line)
{
	int i=0, s=strlen(line);
	const char* tmp=line;
	if (!line) return 0;
	while (s--) {
		tmp++;
		if (isspace(*tmp)) i++; /* don't worry, if user use more than one blank space to sperate file types */
	}
		
	return i+1;
}

int isPicture(const char* path, INFO* info)
{
    char* tmp;
    int i, j;
    if (!path) return 0;
    if (strlen(path) < info->max_type_length) return 0;
    tmp=strrchr(path, '.');
    if (!tmp) return 0;
    j=0;
    for (i=0; (unsigned) i<info->type_count; ++i) 
        j += !strcmp(info->type[i], tmp);
    
    return j;
}

int generateRandom(int left, int right)
{
    int z;
    if (left > right) return 0;
    if (left==right) return left;
    
    srand(time(NULL));

    z = rand()%right+1;

    z += z<left ? left : 0;

    return z;
}

